from typing import Union, Any, Sequence, Callable

import torch
import numpy as np
import torchvision.transforms as T


__all__ = ["Compose", "ToTensor", "ToDtype"]


class Compose(T.Compose):

    def __init__(self, transforms: Sequence[Callable]):
        self.transforms = transforms

    def __call__(self, x: Union[Any, Sequence]):
        if isinstance(x, Sequence):
            for t in self.transforms:
                x = [t(i) for i in x]
        else:
            for t in self.transforms:
                x = t(x)
        return x


class ToTensor(object):

    def __init__(self, permute_dims: bool = True):
        self.permute_dims = permute_dims

    def __call__(self, x: np.ndarray) -> torch.Tensor:
        x = torch.from_numpy(x)

        if x.ndim == 2:
            x = x[:, :, None]

        # Convert HWC->CHW
        if self.permute_dims:
            x = x.permute((2, 0, 1)).contiguous()

        return x


class ToDtype(object):
    
    def __init__(self, dtype: torch.dtype):
        self.dtype = dtype

    def __call__(self, x: torch.Tensor) -> torch.Tensor:
        return x.to(self.dtype)



from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

with open("requirements.txt", "r") as f:
    install_requires = f.read().strip().splitlines()

setup_requires = ["pytest-runner"]
tests_require = ["pytest", "pytest-cov", "mock", "mypy", "black", "pylint"]

url = "https://github.com/isaaccorley/torchrs"
__version__ = "0.0.1"

setup(
    name="torch-rs",
    version="0.0.1",
    license='MIT License',
    author="Isaac Corley",
    author_email="isaac.corley@my.utsa.edu",
    description="PyTorch Library for Remote Sensing",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=url,
    download_url='{}/archive/{}.tar.gz'.format(url, __version__),
    keywords=["pytorch", "remote-sensing", "computer-vision"],
    project_urls={
        "Bug Tracker": "https://github.com/isaaccorley/torchrs/issues",
    },
    install_requires=install_requires,
    setup_requires=setup_requires,
    tests_require=tests_require,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=find_packages(),
    python_requires=">=3.7",
)
